var Ogs = function()
{
	var ogs = {};
	var debug = 0;
	var cluster;
	var cmdHistory = [];
	var cmdIndex = 0;
	
	ogs.initPage = function()
	{
		var url = '../../OpenGrid.servlet?ID='+App.getSessionId();
		url += '&cmd=GetHostInfo';
		url += '&clusterId='+encodeURIComponent(Data.get('page-data', 'cluster-id'));
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, ogs.onHostInfoLoaded, ogs.onHostInfoLoaded);
		request.send(null);
		
		// All other initialization happens after info has been loaded
	}
	
	ogs.onHostInfoLoaded = function(request)
	{
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			alert(error);
			return;
		}

		cluster = response.cluster;
		var info = cluster.hostInfo.info;
		Doc.element('cluster-name').innerHTML = Strings.encodeTags(cluster.name);
		Doc.element('cluster-address').innerHTML = Strings.encodeTags(cluster.address + ':' + cluster.port);
		if (info.exitStatus == 0)
		{
			Doc.element('cluster-info').innerHTML = Strings.encodeTags(info.stdout);
		}
		else
		{
			Doc.element('cluster-info').innerHTML = '<div class="messagecontainer error">['+info.exitStatus+'] '+Strings.encodeTags(info.stderr)+'</div>';
		}
		
		// Clear output window
		ogs.clearOut();
		
		// Add event handlers
		Buttons.addClickHandler('btnClear', ogs.clearOut);
		var cmd = Doc.element('ssh-cmd');
		Events.doOnEnter(cmd, ogs.executeCmd);
		Events.addEventHandler(cmd, 'keypress', ogs.handleCommandHistory);
		
		// Focus the command input
		cmd.focus();
	}
	
	/* Clear the output window */
	ogs.clearOut = function()
	{
		var msg = '<span style="color: #FFFFFF">['+cluster.user + '@' + cluster.id + ']</span>\n';
		msg += 'Type commands below and press ENTER to submit to the cluster.\n---\n';
		Doc.element('ssh-out').innerHTML = msg;
		Doc.element('ssh-cmd').focus();
	}
	
	ogs.executeCmd = function(event)
	{
		var cmd = Doc.element('ssh-cmd');

		cmdHistory[cmdHistory.length] = cmd.value;
		cmdIndex = cmdHistory.length;

		
		var submitInfo = {};
		submitInfo.clusterId = Data.get('page-data', 'cluster-id');
		submitInfo.cmd = cmd.value;
		cmd.value = '';

		// Special intercept not sent to remote host
		if (submitInfo.cmd == 'clear')
		{
			ogs.clearOut();
			return;
		}
		
		var url = '../../OpenGrid.servlet?ID='+App.getSessionId();
		url += '&cmd=ExecuteCmd';
		var request = Ajax.getXmlHttpRequest();
		request.open("POST", url, true);
		Ajax.setReadyStateHandler(request, ogs.onCommandExecuted, ogs.onCommandExecuted);
		request.send(JSON.stringify(submitInfo));
	}
	
	ogs.onCommandExecuted = function(request)
	{
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		var out = Doc.element('ssh-out');
		
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			out.innerHTML += '<span class="stderr">'+Strings.encodeTags(error)+'</span>\n';
			return;
		}
		
		var result = response.result;
		out.innerHTML += '<span class="cmd">' + Strings.encodeTags(response.cmd)+'</span>\n';
		if (result.stdout)
		{
			out.innerHTML += '<span class="stdout">'+Strings.encodeTags(result.stdout)+'</span>\n';
		}
		if (result.stderr)
		{
			out.innerHTML += '<span class="stderr">['+result.exitStatus+'] '+Strings.encodeTags(result.stderr)+'</span>\n';
		}
		out.scrollTop = out.scrollHeight;
		
	}
	
	ogs.handleCommandHistory = function(event)
	{
		var up = event.keyCode == 38;
		var down = event.keyCode == 40;
		
		if (up || down) // up
		{
			var cmd = Doc.element('ssh-cmd');
			// Saves the current cmd if typing has started
			if (cmd.value)
			{
				if (cmdIndex >= 0 && cmdIndex <= cmdHistory.length)
				{
					cmdHistory[cmdIndex] = cmd.value;
				}
			}
			
			if (up && cmdIndex >= 0)
			{
				cmdIndex--;
				Doc.element('ssh-cmd').value = cmdIndex >= 0 ? cmdHistory[cmdIndex] : '';
			}
			else if (down && cmdIndex < cmdHistory.length)
			{
				cmdIndex++;
				Doc.element('ssh-cmd').value = cmdIndex < cmdHistory.length ? cmdHistory[cmdIndex] : '';
			}
		}
	}
	
	return ogs;
}();

Doc.onLoad(Ogs.initPage);
